from __future__ import annotations

from collections.abc import Callable
from re import Pattern
from typing import Generic
from typing_extensions import TypeVarTuple, Unpack

from .config import Config
from .model import Permission, ResourceNode, Role, User
from .service import PermissionService
from .store import BaseStore


class PermissionNotFoundError(KeyError):
    """资源不存在或父资源不存在时抛出。"""

    pass


class PermissionDeniedError(PermissionError):
    """权限不足时抛出。"""

    pass


Ts = TypeVarTuple("Ts")


class PermissionExecutor(Generic[Unpack[Ts]]):
    """
    专门负责对“权限状态”的 get/set 操作。

    这里的“权限状态”定义为：某个 subject 在某个资源节点上的有效权限 bitmask（int）。

    - suget / suset : 以 root 身份操作（无权限检查）
    - get / set     : 以执行者（某个 User）身份操作，并按题设规则做检查
    """

    def __init__(
        self,
        storage: BaseStore,
        perm_service: PermissionService[Unpack[Ts]],
    ):
        self.storage = storage
        self.service = perm_service

    def _get_parent_and_self(
        self,
        resource_path: str,
        missing_ok: bool,
    ) -> tuple[ResourceNode | None, ResourceNode | None]:
        """获取父节点和当前节点。

        Args:
            resource_path (str): 资源路径。
            missing_ok (bool): 是否允许节点不存在。

        Returns:
            tuple[ResourceNode | None, ResourceNode | None]: (parent_node, self_node)。

        Raises:
            PermissionNotFoundError: 当 missing_ok=False 且节点不存在时抛出。
        """
        path = resource_path.strip(Config.NODE_SEPARATOR)
        if not path:
            # 根节点没有父节点的概念，这里约定 parent=None, self=root(如果存在)
            root = self.storage.resources.get("root")
            if root is None and not missing_ok:
                raise PermissionNotFoundError("Root resource 'root' does not exist")
            return None, root

        # 获取 self
        self_node = self.storage.resources.get(path)

        # 获取 parent
        parent_id = None
        if Config.NODE_SEPARATOR in path:
            parent_id, _, _ = path.rpartition(Config.NODE_SEPARATOR)

        parent_node = self.storage.resources.get(parent_id) if parent_id else None

        # 父节点不存在
        if parent_node is None and parent_id and not missing_ok:
            raise PermissionNotFoundError(f"Parent resource '{parent_id}' not found")

        # 自身不存在
        if self_node is None and not missing_ok:
            raise PermissionNotFoundError(f"Resource '{path}' not found")

        return parent_node, self_node

    def _ensure_resource_for_set(
        self,
        resource_path: str,
        missing_ok: bool,
    ) -> tuple[ResourceNode | None, ResourceNode]:
        """确保资源存在以进行设置操作。

        Args:
            resource_path (str): 资源路径。
            missing_ok (bool): 是否允许自动创建。

        Returns:
            tuple[ResourceNode | None, ResourceNode]: (parent_node, self_node)。

        Raises:
            PermissionNotFoundError: 当 missing_ok=False 且资源不存在时抛出。
        """
        parent, node = self._get_parent_and_self(resource_path, missing_ok=missing_ok)
        if node is None:
            if not missing_ok:
                raise PermissionNotFoundError(f"Resource '{resource_path}' not found")
            # 自动创建
            node = self.storage.define(resource_path)
        return parent, node

    def _apply_chmod(
        self,
        old_mask: int,
        mask: int,
        mode: str,
    ) -> int:
        """应用 chmod 风格的权限调整。

        Args:
            old_mask (int): 旧的权限掩码。
            mask (int): 新的权限掩码。
            mode (str): 调整模式。
                "=" : 覆盖为 mask
                "+" : old_mask | mask
                "-" : old_mask & ~mask

        Returns:
            int: 调整后的权限掩码。

        Raises:
            ValueError: 当 mode 不支持时抛出。
        """
        if mode == "=":
            return mask
        elif mode == "+":
            return old_mask | mask
        elif mode == "-":
            return old_mask & ~mask
        else:
            raise ValueError(f"Unsupported chmod mode: {mode!r}")

    def suget(
        self,
        subject: User | Role,
        resource_path: str,
        missing_ok: bool = False,
        context: tuple[Unpack[Ts]] | None = None,
    ) -> int | None:
        """root 获取 subject 在指定节点上的权限状态（bitmask），不做权限校验。

        Args:
            subject (User | Role): 目标主体（用户或角色）。
            resource_path (str): 资源路径。
            missing_ok (bool, optional): 是否允许节点不存在。默认为 False。
            context (tuple | None, optional): 上下文信息。默认为 None。

        Returns:
            int | None: 权限掩码。如果 missing_ok=True 且节点不存在，返回 None。

        Raises:
            PermissionNotFoundError: 当 missing_ok=False 且节点不存在时抛出。
        """
        parent, node = self._get_parent_and_self(resource_path, missing_ok=missing_ok)
        # 若 missing_ok=True，则 parent 或 node 可能为 None，此时按规则返回 None
        if node is None:
            return None

        context = context or tuple()

        # 直接用内部方法计算该 subject 的静态权限
        if isinstance(subject, Role):
            cache: dict[tuple[str, str, str], int] = {}
            mask = self.service._calc_permissions_for_subject(
                subject.type,
                subject.id,
                node,
                context,
                visited=[],
                cache=cache,
            )
        else:
            mask = self.service.get_effective_permissions(subject, node.id, context)
        return mask

    def test(
        self,
        subject: User | Role,
        resource_path: str,
        required_mask: int,
        missing_ok: bool = False,
        context: tuple[Unpack[Ts]] | None = None,
    ) -> bool:
        """root 测试 subject 在指定节点上是否拥有某些权限，不做权限校验。

        Args:
            subject (User | Role): 目标主体（用户或角色）。
            resource_path (str): 资源路径。
            required_mask (int): 需要的权限掩码。
            missing_ok (bool, optional): 是否允许节点不存在。默认为 False。
            context (tuple | None, optional): 上下文信息。默认为 None。

        Returns:
            bool: 是否拥有指定权限。如果 missing_ok=True 且节点不存在，返回 False。

        Raises:
            PermissionNotFoundError: 当 missing_ok=False 且节点不存在时抛出。
        """
        mask = self.suget(
            subject,
            resource_path,
            missing_ok=missing_ok,
            context=context,
        )
        if mask is None:
            mask = Permission.VISIT | Permission.AVAILABLE
        return (mask & required_mask) == required_mask

    def get(
        self,
        executor: User,
        resource_path: str,
        missing_ok: bool = False,
        context: tuple[Unpack[Ts]] | None = None,
    ) -> int | None:
        """执行者获取自己在目标节点的权限状态。

        Args:
            executor (User): 执行者。
            resource_path (str): 资源路径。
            missing_ok (bool, optional): 是否允许节点不存在。默认为 False。
            context (tuple | None, optional): 上下文信息。默认为 None。

        Returns:
            int | None: 权限掩码。如果 missing_ok=True 且节点不存在，返回 None。

        Raises:
            PermissionNotFoundError: 当 missing_ok=False 且节点不存在时抛出。
            PermissionDeniedError: 当执行者在父节点缺少 VISIT+AVAILABLE 权限，或在自身缺少 VISIT 权限时抛出。
        """
        parent, node = self._get_parent_and_self(resource_path, missing_ok=missing_ok)
        context = context or tuple()

        # 2. 检查执行者在父节点是否有 v+a（VISIT + AVAILABLE）
        if parent is not None:
            parent_mask = self.service.get_effective_permissions(executor, parent.id, context)
            required_parent = Permission.VISIT | Permission.AVAILABLE
            if (parent_mask & required_parent) != required_parent:
                raise PermissionDeniedError(f"Executor '{executor.id}' lacks VISIT+AVAILABLE on parent '{parent.id}'")

        # 3. 自身不存在
        if node is None:
            return None

        # 4. 检查自身权限包含 VISIT
        self_mask = self.service.get_effective_permissions(executor, node.id, context)
        if (self_mask & Permission.VISIT) != Permission.VISIT:
            raise PermissionDeniedError(f"Executor '{executor.id}' lacks VISIT on '{node.id}'")
        return self_mask

    def suset(
        self,
        subject: User | Role,
        resource_path: str | Callable[[str], bool] | Pattern[str],
        mask: int,
        mode: str = "=",
        missing_ok: bool = False,
    ) -> None:
        """root 为 subject 在指定节点上“设置权限状态”。

        Args:
            subject (User | Role): 目标主体（用户或角色）。
            resource_path (str | Callable[[str], bool] | Pattern[str]): 资源路径或匹配模式。
            mask (int): 权限掩码。
            mode (str, optional): 设置模式 ("=", "+", "-")。默认为 "="。
            missing_ok (bool, optional): 是否允许节点不存在。默认为 False。

        Raises:
            PermissionNotFoundError: 当 missing_ok=False 且节点不存在时抛出。
            ValueError: 当 mode 不支持时抛出。
        """

        if isinstance(resource_path, str):
            resource_path = resource_path.strip(Config.NODE_SEPARATOR)
            if "*" in resource_path or "?" in resource_path or "[" in resource_path:
                matched = self.storage.glob_resources(resource_path)
            else:
                _, node = self._ensure_resource_for_set(resource_path, missing_ok=missing_ok)

                primary_acl = self.storage.get_primary_acl(subject, node.id)
                old_mask = primary_acl.allow_mask if primary_acl else 0
                new_mask = self._apply_chmod(old_mask, mask, mode)
                if primary_acl:
                    # 更新已有 ACL
                    primary_acl.allow_mask = new_mask
                else:
                    # 创建新 ACL
                    self.storage.assign(
                        subject=subject,
                        resource_path=node.id,
                        allow_mask=new_mask,
                    )
                return
        elif isinstance(resource_path, Pattern):
            matched = self.storage.match_resources(lambda t: bool(resource_path.fullmatch(t)))
        else:
            matched = self.storage.match_resources(resource_path)
        for node in matched:
            if node.parent_id and node.parent_id not in self.storage.resources:
                if not missing_ok:
                    raise PermissionNotFoundError(f"Parent '{node.parent_id}' for '{node.id}' not found")
                else:
                    continue

            primary_acl = self.storage.get_primary_acl(subject, node.id)
            old_mask = primary_acl.allow_mask if primary_acl else 0
            new_mask = self._apply_chmod(old_mask, mask, mode)

            if primary_acl is None:
                self.storage.assign(
                    subject=subject,
                    resource_path=node.id,
                    allow_mask=new_mask,
                )
            else:
                primary_acl.allow_mask = new_mask

    def set(
        self,
        executor: User,
        target: User | Role,
        resource_path: str | Callable[[str], bool] | Pattern[str],
        mask: int,
        mode: str = "=",
        missing_ok: bool = False,
        context: tuple[Unpack[Ts]] | None = None,
    ) -> None:
        """执行者为目标 subject 设置目标节点的权限状态。

        Args:
            executor (User): 执行者。
            target (User | Role): 目标主体（用户或角色）。
            resource_path (str | Callable[[str], bool] | Pattern[str]): 资源路径或匹配模式。
            mask (int): 权限掩码。
            mode (str, optional): 设置模式 ("=", "+", "-")。默认为 "="。
            missing_ok (bool, optional): 是否允许节点不存在。默认为 False。
            context (tuple | None, optional): 上下文信息。默认为 None。

        Raises:
            PermissionNotFoundError: 当 missing_ok=False 且节点不存在时抛出。
            PermissionDeniedError: 当执行者权限不足时抛出。
            ValueError: 当 mode 不支持时抛出。
        """
        context = context or tuple()

        if isinstance(resource_path, str):
            resource_path = resource_path.strip(Config.NODE_SEPARATOR)
            if "*" in resource_path or "?" in resource_path or "[" in resource_path:
                matched = self.storage.glob_resources(resource_path)
            else:
                parent, node = self._ensure_resource_for_set(resource_path, missing_ok=missing_ok)

                # 2. 检查执行者在父节点是否有 v+m+a
                if parent is not None:
                    parent_mask = self.service.get_effective_permissions(executor, parent.id, context)
                    required_parent = Permission.VISIT | Permission.MODIFY | Permission.AVAILABLE
                    if (parent_mask & required_parent) != required_parent:
                        raise PermissionDeniedError(f"Executor '{executor.id}' lacks V+M+A on parent '{parent.id}'")

                # 4. 检查执行者在自身是否有 MODIFY
                self_mask = self.service.get_effective_permissions(executor, node.id, context)
                if (self_mask & Permission.MODIFY) != Permission.MODIFY:
                    # 不修改该节点状态
                    return

                primary_acl = self.storage.get_primary_acl(target, node.id)
                old_mask = primary_acl.allow_mask if primary_acl else 0
                new_mask = self._apply_chmod(old_mask, mask, mode)
                if primary_acl:
                    # 更新已有 ACL
                    primary_acl.allow_mask = new_mask
                else:
                    # 创建新 ACL
                    self.storage.assign(
                        subject=target,
                        resource_path=node.id,
                        allow_mask=new_mask,
                    )
                return
        elif isinstance(resource_path, Pattern):
            matched = self.storage.match_resources(lambda t: bool(resource_path.fullmatch(t)))
        else:
            matched = self.storage.match_resources(resource_path)
        for node in matched:
            parent = self.storage.resources.get(node.parent_id) if node.parent_id else None
            if node.parent_id and parent is None:
                if not missing_ok:
                    raise PermissionNotFoundError(f"Parent '{node.parent_id}' for '{node.id}' not found")
                else:
                    continue
            if parent is not None:
                parent_mask = self.service.get_effective_permissions(executor, parent.id, context)
                required_parent = Permission.VISIT | Permission.MODIFY | Permission.AVAILABLE
                if (parent_mask & required_parent) != required_parent:
                    raise PermissionDeniedError(f"Executor '{executor.id}' lacks V+M+A on parent '{parent.id}'")
            self_mask = self.service.get_effective_permissions(executor, node.id, context)
            if (self_mask & Permission.MODIFY) != Permission.MODIFY:
                continue  # 无修改权限，跳过

            # 5. chmod 更新目标 subject
            primary_acl = self.storage.get_primary_acl(target, node.id)
            old_mask = primary_acl.allow_mask if primary_acl else 0
            new_mask = self._apply_chmod(old_mask, mask, mode)
            if primary_acl:
                # 更新已有 ACL
                primary_acl.allow_mask = new_mask
            else:
                # 创建新 ACL
                self.storage.assign(
                    subject=target,
                    resource_path=node.id,
                    allow_mask=new_mask,
                )



from __future__ import annotations

from collections.abc import Callable
from typing import Generic, Protocol
from typing_extensions import TypeVarTuple, Unpack

from .model import ResourceNode, Role, User

Ts = TypeVarTuple("Ts")


class PermissionStrategy(Protocol[Unpack[Ts]]):
    """权限策略协议。"""

    def __call__(
        self,
        user: User,
        resource: ResourceNode,
        context: tuple[Unpack[Ts]],
        current_mask: int,
        permission_lookup: Callable[[User | Role, Unpack[Ts]], int],
    ) -> int:
        """执行策略。

        Args:
            user (User): 用户对象。
            resource (ResourceNode): 资源节点。
            context (tuple[Unpack[Ts]]): 上下文信息。
            current_mask (int): 当前权限掩码。
            permission_lookup (Callable[[User | Role, Unpack[Ts]], int]): 权限查找回调函数。

        Returns:
            int: 更新后的权限掩码。
        """
        ...


class PermissionEngine(Generic[Unpack[Ts]]):
    """权限引擎，管理和应用权限策略。"""

    def __init__(self):
        self._strategies: list[PermissionStrategy[Unpack[Ts]]] = []

    def register_strategy(self, strategy: PermissionStrategy[Unpack[Ts]]):
        """注册策略。

        Args:
            strategy (PermissionStrategy[Unpack[Ts]]): 策略对象。
        """
        self._strategies.append(strategy)

    def apply_strategies(
        self,
        user: User,
        resource: ResourceNode,
        context: tuple[Unpack[Ts]],
        mask: int,
        permission_lookup: Callable[[User | Role, Unpack[Ts]], int],
    ) -> int:
        """应用所有注册的策略。

        Args:
            user (User): 用户对象。
            resource (ResourceNode): 资源节点。
            context (tuple[Unpack[Ts]]): 上下文信息。
            mask (int): 初始权限掩码。
            permission_lookup (Callable[[User | Role, Unpack[Ts]], int]): 权限查找回调函数。

        Returns:
            int: 最终权限掩码。
        """
        for s in self._strategies:
            mask = s(user, resource, context, mask, permission_lookup)
        return mask



from __future__ import annotations

import os
import sqlite3
from pathlib import Path

from arclet.cithun import InheritMode
from arclet.cithun.model import AclDependency, AclEntry, ResourceNode, Role, Track, TrackLevel, User
from arclet.cithun.store import BaseStore


class SimpleDatabaseStore(BaseStore):
    def __init__(self, file: os.PathLike[str]):
        file = Path(file)
        self.file = file
        self.conn = sqlite3.connect(self.file)
        super().__init__()
        self.ensure_table()

    def ensure_table(self):
        sql = """\
        CREATE TABLE IF NOT EXISTS users (
            id TEXT PRIMARY KEY,
            name TEXT NOT NULL
        );
        CREATE TABLE IF NOT EXISTS roles (
            id TEXT PRIMARY KEY,
            name TEXT NOT NULL
        );

        CREATE TABLE IF NOT EXISTS user_roles (
            user_id TEXT,
            role_id TEXT,
            PRIMARY KEY (user_id, role_id),
            FOREIGN KEY (user_id) REFERENCES users(id) ON DELETE CASCADE,
            FOREIGN KEY (role_id) REFERENCES roles(id) ON DELETE CASCADE
        );

        CREATE TABLE IF NOT EXISTS role_inherits (
            role_id TEXT,
            parent_role_id TEXT,
            PRIMARY KEY (role_id, parent_role_id),
            FOREIGN KEY (role_id) REFERENCES roles(id) ON DELETE CASCADE,
            FOREIGN KEY (parent_role_id) REFERENCES roles(id) ON DELETE CASCADE
        );

        CREATE TABLE IF NOT EXISTS resources (
            id TEXT PRIMARY KEY,
            name TEXT NOT NULL,
            parent_id TEXT NULL,
            inherit_mode INTEGER DEFAULT 0,  -- 0: INHERIT, 1: MERGE, 2: OVERRIDE
            type TEXT NOT NULL DEFAULT 'GENERIC',  -- FILE / DIR / PROJECT / etc.
            FOREIGN KEY (parent_id) REFERENCES resources(id) ON DELETE CASCADE
        );
        CREATE INDEX IF NOT EXISTS idx_resources_parent_id ON resources(parent_id);

        CREATE TABLE IF NOT EXISTS acls (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            subject_type TEXT NOT NULL,  -- 'USER' or 'ROLE'
            subject_id TEXT NOT NULL,
            resource_id TEXT NOT NULL,
            allow_mask INTEGER NOT NULL,
            deny_mask INTEGER DEFAULT 0,
            FOREIGN KEY (resource_id) REFERENCES resources(id) ON DELETE CASCADE
        );

        CREATE INDEX IF NOT EXISTS idx_acls_resource_id ON acls(resource_id);
        CREATE INDEX IF NOT EXISTS idx_acls_subject ON acls(subject_type, subject_id);

        CREATE TABLE IF NOT EXISTS acl_dependencies (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            acl_id INTEGER NOT NULL,
            dep_subject_type TEXT NOT NULL,
            dep_subject_id TEXT NOT NULL,
            dep_resource_id TEXT NOT NULL,
            required_mask INTEGER NOT NULL,
            FOREIGN KEY (acl_id) REFERENCES acls(id) ON DELETE CASCADE,
            FOREIGN KEY (dep_resource_id) REFERENCES resources(id) ON DELETE CASCADE
        );

        CREATE INDEX IF NOT EXISTS idx_acl_deps_acl_id  ON acl_dependencies(acl_id);
        CREATE INDEX IF NOT EXISTS idx_acl_deps_dep_subject ON acl_dependencies(dep_subject_type, dep_subject_id);
        CREATE INDEX IF NOT EXISTS idx_acl_deps_dep_resource ON acl_dependencies(dep_resource_id);

        CREATE TABLE IF NOT EXISTS tracks (
            id TEXT PRIMARY KEY,
            name TEXT NOT NULL
        );
        CREATE TABLE IF NOT EXISTS track_levels (
            `index` INTEGER,
            track_id TEXT,
            role_id TEXT,
            level_name TEXT NOT NULL,
            PRIMARY KEY (track_id, role_id),
            FOREIGN KEY (track_id) REFERENCES tracks(id) ON DELETE CASCADE,
            FOREIGN KEY (role_id) REFERENCES roles(id) ON DELETE CASCADE
        );
        CREATE INDEX IF NOT EXISTS idx_track_levels_track_id ON track_levels(track_id);
        CREATE INDEX IF NOT EXISTS idx_track_levels_role_id ON track_levels(role_id);
        """
        self.conn.executescript(sql)
        self.conn.commit()

    def create_user(self, uid: str, name: str) -> User:
        if uid in self.users:
            return self.users[uid]
        user = User(uid, name)
        self.users[uid] = user
        cursor = self.conn.cursor()
        cursor.execute("INSERT INTO users (id, name) VALUES (?, ?);", (user.id, user.name))
        self.conn.commit()
        return user

    def create_role(self, rid: str, name: str) -> Role:
        if rid in self.roles:
            return self.roles[rid]
        role = Role(rid, name)
        self.roles[rid] = role
        cursor = self.conn.cursor()
        cursor.execute("INSERT INTO roles (id, name) VALUES (?, ?);", (role.id, role.name))
        self.conn.commit()
        return role

    def create_track(self, tid: str, name: str | None = None) -> Track:
        if tid in self.tracks:
            return self.tracks[tid]
        track = Track(tid, name or tid)
        self.tracks[tid] = track
        cursor = self.conn.cursor()
        cursor.execute("INSERT INTO tracks (id, name) VALUES (?, ?);", (track.id, track.name))
        self.conn.commit()
        return track

    def _add_acl(self, acl: AclEntry):
        target_acl = next(
            (
                i
                for i in self.acls
                if i.subject_type == acl.subject_type
                and i.subject_id == acl.subject_id
                and i.resource_id == acl.resource_id
            ),
            None,
        )
        if target_acl:
            return target_acl
        cursor = self.conn.cursor()
        cursor.execute(
            "INSERT INTO acls (subject_type, subject_id, resource_id, allow_mask, deny_mask) "
            "VALUES (?, ?, ?, ?, ?);",
            (acl.subject_type.value, acl.subject_id, acl.resource_id, acl.allow_mask, acl.deny_mask),
        )
        self.conn.commit()
        self.acls.append(acl)

    def define(
        self,
        path: str,
        inherit_mode: InheritMode | None = None,
        type_: str = "GENERIC",
    ) -> ResourceNode:
        old_keys = set(self.resources.keys())
        resource = super().define(path, inherit_mode, type_)
        additional_keys = set(self.resources.keys()) - old_keys
        cursor = self.conn.cursor()
        for rid in additional_keys:
            res = self.resources[rid]
            cursor.execute(
                "INSERT INTO resources (id, name, parent_id, inherit_mode, type) " "VALUES (?, ?, ?, ?, ?);",
                (res.id, res.name, res.parent_id, res.inherit_mode.value, res.type),
            )
        self.conn.commit()
        return resource

    def depend(
        self,
        target_subject: User | Role,
        target_resource_id: str,
        dep_subject: User | Role,
        dep_resource_path: str,
        required_mask: int,
    ) -> AclEntry:
        target_acl = self.get_primary_acl(target_subject, target_resource_id)
        if not target_acl:
            raise ValueError("Target ACL does not exist.")
        dep_res = self.define(dep_resource_path)
        dep = AclDependency(
            subject_type=dep_subject.type,
            subject_id=dep_subject.id,
            resource_id=dep_res.id,
            required_mask=required_mask,
        )
        if dep in target_acl.dependencies:
            return target_acl
        target_acl.dependencies.append(dep)

        cursor = self.conn.cursor()
        cursor.execute(
            "SELECT id FROM acls WHERE subject_type = ? AND subject_id = ? AND resource_id = ?;",
            (target_acl.subject_type.value, target_acl.subject_id, target_acl.resource_id),
        )
        row = cursor.fetchone()
        if row is None:
            raise ValueError("ACL entry not found after creation.")
        acl_id = row[0]

        cursor.execute(
            "INSERT INTO acl_dependencies (acl_id, dep_subject_type, dep_subject_id, "
            "dep_resource_id, required_mask) VALUES (?, ?, ?, ?, ?);",
            (acl_id, dep.subject_type.value, dep.subject_id, dep.resource_id, dep.required_mask),
        )
        self.conn.commit()
        return target_acl

    def inherit(self, child: User | Role, parent: Role):
        if isinstance(child, Role):
            child_role = self._ensure_role(child)
            self._ensure_role(parent)
            if parent.id not in child_role.parent_role_ids:
                cursor = self.conn.cursor()
                cursor.execute(
                    "INSERT INTO role_inherits (role_id, parent_role_id) VALUES (?, ?);", (child_role.id, parent.id)
                )
                self.conn.commit()
                child_role.parent_role_ids.append(parent.id)
        else:
            user = self._ensure_user(child)
            self._ensure_role(parent)
            if parent.id not in user.role_ids:
                cursor = self.conn.cursor()
                cursor.execute("INSERT INTO user_roles (user_id, role_id) VALUES (?, ?);", (user.id, parent.id))
                self.conn.commit()
                user.role_ids.append(parent.id)

    def add_track_level(self, track: Track, role: Role, name: str | None = None) -> None:
        level = TrackLevel(role.id, name or role.name)
        if level in track.levels:
            return
        track.levels.append(level)
        cursor = self.conn.cursor()
        cursor.execute(
            "INSERT INTO track_levels (`index`, track_id, role_id, level_name) VALUES (?, ?, ?, ?);",
            (len(track.levels) - 1, track.id, role.id, level.level_name),
        )
        self.conn.commit()

    def insert_track_level(self, track: Track, index: int, role: Role, name: str | None = None) -> None:
        level = TrackLevel(role.id, name or role.name)
        if level in track.levels:
            return
        track.levels.insert(index, level)
        cursor = self.conn.cursor()
        cursor.execute("DELETE FROM track_levels WHERE track_id = ?;", (track.id,))
        for idx, lvl in enumerate(track.levels):
            cursor.execute(
                "INSERT INTO track_levels (`index`, track_id, role_id, level_name) VALUES (?, ?, ?, ?);",
                (idx, track.id, lvl.role_id, lvl.level_name),
            )
        self.conn.commit()

    def load(self):
        cursor = self.conn.cursor()
        cursor.execute("SELECT id, name FROM users;")
        for row in cursor.fetchall():
            user = User(*row, role_ids=[])
            self.users[user.id] = user
        cursor.execute("SELECT id, name FROM roles;")
        for row in cursor.fetchall():
            role = Role(*row, parent_role_ids=[])
            self.roles[role.id] = role
        for user in self.users.values():
            cursor.execute("SELECT role_id FROM user_roles WHERE user_id = ?;", (user.id,))
            role_ids = [r[0] for r in cursor.fetchall()]
            user.role_ids.extend(role_ids)
        for role in self.roles.values():
            cursor.execute("SELECT parent_role_id FROM role_inherits WHERE role_id = ?;", (role.id,))
            parent_role_ids = [r[0] for r in cursor.fetchall()]
            role.parent_role_ids.extend(parent_role_ids)
        cursor.execute("SELECT id, name, parent_id, inherit_mode, type FROM resources;")
        for row in cursor.fetchall():
            resource = ResourceNode(*row)
            self.resources[resource.id] = resource
        cursor.execute("SELECT id, subject_type, subject_id, resource_id, allow_mask, deny_mask FROM acls;")
        acl_ids = []
        for row in cursor.fetchall():
            acl = AclEntry(*row[1:], dependencies=[])
            self.acls.append(acl)
            acl_ids.append(row[0])
        for acl, acl_id in zip(self.acls, acl_ids):
            cursor.execute(
                "SELECT dep_subject_type, dep_subject_id, dep_resource_id, required_mask "
                "FROM acl_dependencies WHERE acl_id = ?;",
                (acl_id,),
            )
            for dep_row in cursor.fetchall():
                dep = AclDependency(*dep_row)
                acl.dependencies.append(dep)
        cursor.execute("SELECT id, name FROM tracks;")
        for row in cursor.fetchall():
            track = Track(*row, levels=[])
            self.tracks[track.id] = track
        for track in self.tracks.values():
            cursor.execute(
                "SELECT role_id, level_name FROM track_levels WHERE track_id = ? ORDER BY `index` ASC;", (track.id,)
            )
            for level_row in cursor.fetchall():
                level = TrackLevel(*level_row)
                track.levels.append(level)

    def save(self):
        pass
